@@ -12,16 +12,15 @@ use syn::{
     GenericArgument, Lit, Meta, NestedMeta, PathArguments, Type,
 };
 
-use abigen::{expand, ContractArgs};
+use abigen::Contracts;
 use ethers_core::abi::{param_type::Reader, AbiParser, Event, EventExt, EventParam, ParamType};
 use hex::FromHex;
-use spanned::Spanned;
 
 mod abigen;
 mod spanned;
 
-/// Proc macro to generate type-safe bindings to a contract. This macro accepts
-/// an Ethereum contract ABI or a path. Note that this path is rooted in
+/// Proc macro to generate type-safe bindings to a contract(s). This macro accepts
+/// one or more Ethereum contract ABI or a path. Note that this path is rooted in
 /// the crate's root `CARGO_MANIFEST_DIR`.
 ///
 /// # Examples
@@ -55,6 +54,8 @@ mod spanned;
 /// - `event_derives`: A list of additional derives that should be added to
 ///   contract event structs and enums.
 ///
+/// # Example
+///
 /// ```ignore
 /// abigen!(
 ///     MyContract,
@@ -65,13 +66,32 @@ mod spanned;
 ///     event_derives (serde::Deserialize, serde::Serialize),
 /// );
 /// ```
+///
+/// `abigen!` supports multiple abigen definitions separated by a semicolon `;`
+/// This is useful if the contracts use ABIEncoderV2 structs. In which case `abigen!` bundles all type duplicates so that all rust contracts also use the same rust types.
+///
+/// # Example Multiple contracts
+/// ```ignore
+/// abigen!(
+///     MyContract,
+///     "path/to/MyContract.json",
+///     methods {
+///         myMethod(uint256,bool) as my_renamed_method;
+///     },
+///     event_derives (serde::Deserialize, serde::Serialize);
+///
+///     MyOtherContract,
+///     "path/to/MyOtherContract.json",
+///     event_derives (serde::Deserialize, serde::Serialize);
+/// );
+/// ```
 #[proc_macro]
 pub fn abigen(input: TokenStream) -> TokenStream {
-    let args = parse_macro_input!(input as Spanned<ContractArgs>);
+    let contracts = parse_macro_input!(input as Contracts);
 
-    let span = args.span();
-    expand(args.into_inner())
-        .unwrap_or_else(|e| Error::new(span, format!("{:?}", e)).to_compile_error())
+    contracts
+        .expand()
+        .unwrap_or_else(|err| err.to_compile_error())
         .into()
 }
 
@@ -414,7 +434,7 @@ fn derive_decode_from_log_impl(
         .filter(|f| !f.is_indexed())
         .map(|f| param_type_quote(&f.param.kind));
 
-    let data_types_init = quote! {let data_types = ::std::vec![#( #data_types ),*];};
+    let data_types_init = quote! {let data_types = [#( #data_types ),*];};
 
     // decode
     let (signature_check, flat_topics_init, topic_tokens_len_check) = if event.anonymous {
@@ -478,7 +498,6 @@ fn derive_decode_from_log_impl(
             #( tokens.push(#swap_tokens); )*
         }
     };
-
     Ok(quote! {
 
         let #core_crate::abi::RawLog {data, topics} = log;
@@ -492,7 +511,7 @@ fn derive_decode_from_log_impl(
 
         #tokens_init
 
-        #core_crate::abi::Detokenize::from_tokens(tokens).map_err(|_|#core_crate::abi::Error::InvalidData)
+        #core_crate::abi::Tokenizable::from_token(#core_crate::abi::Token::Tuple(tokens)).map_err(|_|#core_crate::abi::Error::InvalidData)
     })
 }
 
@@ -808,26 +827,6 @@ fn derive_tokenizeable_impl(input: &DeriveInput) -> proc_macro2::TokenStream {
                 #core_crate::abi::Token::Tuple(Vec::new())
             },
         ),
-        1 => {
-            // This is a hacky solution in order to keep the same tokenstream as for tuples
-            let from_token = quote! {
-                let mut iter = Some(token).into_iter();
-                Ok(#init_struct_impl)
-            };
-
-            // This is a hack to get rid of the trailing comma introduced in the macro that concatenates all the fields
-            if let Ok(into_token) = into_token_impl
-                .to_string()
-                .as_str()
-                .trim_end_matches(',')
-                .parse()
-            {
-                (from_token, into_token)
-            } else {
-                return Error::new(input.span(), "Failed to derive Tokenizeable implementation")
-                    .to_compile_error();
-            }
-        }
         _ => {
             let from_token = quote! {
                 if let #core_crate::abi::Token::Tuple(tokens) = token {
